#include "controlworker.h"
#include "frontend_nodes.h"
#include "frontend_param.h"

ControlWorker::ControlWorker(QObject *parent) :
    QObject(parent), conn(NULL)
{

}

ControlWorker::ControlWorker() :
    QObject(), conn(NULL)
{
}

ControlWorker::~ControlWorker()
{
    disconnectBackend();
}

void ControlWorker::connectBackend()
{
    conn = ucart_backendConnect();
}

void ControlWorker::disconnectBackend()
{
    if (conn) {
         ucart_backendDisconnect(conn);
         conn = NULL;
    }
}

ControlWorker::getNodes()
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);
        QStringList nodes;
        for (size_t i = 0; i < num_nodes; i++) {
            nodes << QString(nd[i].name);
        }
        frontend_free_node_data(nd, num_nodes);
        emit(gotNodes(nodes));
    }
}

ControlWorker::getParams(QString node)
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);

        for (size_t i = 0; i < num_nodes; i++) {
            if (QString(nd[i].name) == node) {
                QStringList params;
                // TODO: Get list of params based on node type
                emit(gotParams(params));
            }
        }
        fontend_free_node_data(nd, num_nodes);
    }
}


ControlWorker::getParamValue(QString node, QString param)
{
    if (conn) {
        frontend_node_data * nd = NULL;
        size_t num_nodes = 0;
        frontend_getnodes(conn, &nd, &num_nodes);

        for (size_t i = 0; i < num_nodes; i++) {
            if (QString(nd[i].name) == node) {
                frontend_param_data pd;
                pd.block = nd[i].block;
                pd.param = 0; //TODO: Map string param to param ID based on block type? (Might not be needed...)
                frontend_getparam(conn, &pd);
                emit(gotParamValue(pd.value));
            }
        }
        fontend_free_node_data(nd, num_nodes);
    }
}
